local NekoLoot = LibStub("AceAddon-3.0"):GetAddon("NekoLoot")
local L = LibStub("AceLocale-3.0"):GetLocale("NekoLoot")

local AceDBOpt = LibStub("AceDBOptions-3.0")
local AceConf = LibStub("AceConfig-3.0")
local AceConfReg = LibStub("AceConfigRegistry-3.0")
local AceConfDialog = LibStub("AceConfigDialog-3.0")

local options, moduleOptions = nil, {}
local qualityKeys = {}

------
-- Table of valid anchor keys.
NekoLoot.anchorKeys = {
	TOPRIGHT = "TOPRIGHT", TOP = "TOP", TOPLEFT = "TOPLEFT",
	RIGHT = "RIGHT", CENTER = "CENTER", LEFT = "LEFT", 
	BOTTOMRIGHT = "BOTTOMRIGHT", BOTTOM = "BOTTOM", BOTTOMLEFT = "BOTTOMLEFT"
}


------
-- Table of valid strata keys.
NekoLoot.strataKeys = {
	BACKGROUND = "BACKGROUND", LOW = "LOW", MEDIUM = "MEDIUM",
	HIGH = "HIGH", DIALOG = "DIALOG", FULLSCREEN = "FULLSCREEN",
	FULLSCREEN_DIALOG = "FULLSCREEN_DIALOG", TOOLTIP = "TOOLTIP"
}


------
-- Table of valid font flags
-- MONOCHROME - Font is rendered without antialiasing
-- OUTLINE - Font is displayed with a black outline
-- THICKOUTLINE - Font is displayed with a thick black outline
NekoLoot.fontFlags = { [""] = L["None"], ["MONOCHROME"] = L["Monochrome"], ["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["ThickOutline"], ["OUTLINE, MONOCHROME"] = L["OutlineMono"], ["THICKOUTLINE, MONOCHROME"] = L["ThickOutlineMono"]
}

------
-- Get and set magic
local getOpt, setOpt, getColor, setColor
do
	function getOpt(info)
		-- db.profile.GROUP.KEY --> GROUP is penultimate and KEY is ultimate arg of info
		return NekoLoot.db.profile[info[#info-1]][info[#info]]
	end
	
	function setOpt(info, value)
		-- db.profile.GROUP.KEY --> GROUP is penultimate and KEY is ultimate arg of info
		NekoLoot.db.profile[info[#info-1]][info[#info]] = value
		NekoLoot:ApplySettings()
	end
	
	function getColor(info)
		return unpack(getOpt(info))
	end
	
	function setColor(info, r, g, b, a)
		setOpt(info, {r, g, b, a})
	end
end


------
-- Get options table. If not already builded, create it.
local function getOptions()
	if not options then
		options = {
			type = "group", childGroups = "tab",
			args = {
				frame = {
					type = "group", inline = false,
					name = L["Config/Frame"],
					desc = L["Config/FrameDesc"],
					args = {
						width = {
							type = "range", order = 110,
							name = L["Width"],
							min = 50, max = 400, step = 1,
							get = getOpt,
							set = setOpt,
						},
						scale = {
							type = "range", order = 111,
							name = L["Scale"],
							desc = L["ScaleDesc"],
							min = 0.2, max = 2, step = 0.1,
							get = getOpt,
							set = setOpt,
						},
						alpha = {
							type = "range", order = 112,
							name = L["Alpha"],
							desc = L["AlphaDesc"],
							min = 0.1, max = 1, step = 0.1,
							get = getOpt,
							set = setOpt,
						},
						strata = {
							type = "select", order = 113, style = "dropdown",
							name = L["Strata"],
							desc = L["StrataDesc"],
							values = NekoLoot.strataKeys,
							get = getOpt,
							set = setOpt,
						},

						alignmentHeader = {
							type = "header", order = 120,
							name = L["PositionAlignment"],
						},
						parentAnchor = {
							type = "select", order = 121, style = "dropdown",
							name = L["ParentAnchor"],
							desc = L["ParentAnchorDesc"],
							values = NekoLoot.anchorKeys,
							get = getOpt,
							set = setOpt,
						},
						anchorPoint = {
							type = "select", order = 122, style = "dropdown",
							name = L["AnchorPoint"],
							desc = L["AnchorPointDesc"],
							values = NekoLoot.anchorKeys,
							get = getOpt,
							set = setOpt,
						},

						lockSpace = {
							type = "description", order = 125,
							name = "",
						},
						toggleLock = {
							type = "execute", order = 126,
							name = L["ToggleFrameLock"],
							desc = L["ToggleFrameLockDesc"],
							func = function()
								NekoLoot:ToggleLock(true)
							end,
						},
						lock = {
							type = "toggle", order = 127,
							name = L["Lock"],
							desc = L["LockDesc"],
							get = getOpt,
							set = setOpt,
						},
						
						offsetSpace = {
							type = "description", order = 140,
							name = "",
						},
						xOffSet = {
							type = "range", order = 141, width = "double",
							name = L["XOff"],
							desc = L["XOffDesc"],
							min = -800, max = 800, step = 1,
							get = getOpt,
							set = setOpt,
						},
						yOffSet = {
							type = "range", order = 142, width = "double",
							name = L["YOff"],
							desc = L["YOffDesc"],
							min = -800, max = 800, step = 1,
							get = getOpt,
							set = setOpt,
						},
						
						colorHeader = {
							type = "header", order = 180,
							name = L["Colors"],
						},
						backgroundColor = {
							type = "color", order = 181, hasAlpha = true,
							name = L["BackgroundColor"],
							get = getColor,
							set = setColor,
						},
						borderColor = {
							type = "color", order = 182, hasAlpha = true,
							name = L["BorderColor"],
							get = getColor,
							set = setColor,
						},
					},
				},
				---- frame
			
				loot = {
					type = "group", inline = false,
					name = L["Config/Loot"],
					desc = L["Config/LootDesc"],
					args = {
						descHeader = {
							type = "header", order = 200,
							name = L["ItemDesc"]
						},
						descToggle = {
							type = "toggle", order = 201, width = "double",
							name = L["Toggle"],
							get = getOpt,
							set = setOpt,
						},
						descColor = {
							type = "color", order = 202, width = "double",
							name = L["DescFontColor"],
							desc = L["DescFontColorDesc"],
							get = getColor,
							set = setColor,
						},
						
						headerFontSizes = {
							type = "header", order = 210,
							name = L["FontSizes"],
						},
						descFontSize = {
							type = "range", order = 211,
							name = L["ItemDesc"],
							min = 8, max = 18, step = 1,
							get = getOpt,
							set = setOpt,
						},
						nameFontSize = {
							type = "range", order = 212,
							name = L["ItemName"],
							min = 8, max = 18, step = 1,
							get = getOpt,
							set = setOpt,
						},
						bindFontSize = {
							type = "range", order = 213,
							name = L["ItemBind"],
							desc = L["ItemBindDesc"],
							min = 8, max = 18, step = 1,
							get = getOpt,
							set = setOpt,
						},

						
						widthHeader = {
							type = "header", order = 220,
							name = L["FrameWidth"],
						},
						expand = {
							type = "toggle", order = 221, width = "double",
							name = L["Config/Loot/Expand"],
							desc = L["Config/Loot/ExpandDesc"],
							get = getOpt,
							set = setOpt,
						},
						textWidth = {
							type = "range", order = 222,
							name = L["TextWidth"],
							desc = L["Config/Loot/TextWidthDesc"],
							min = 50, max = 1000, step = 1,
							get = getOpt,
							set = setOpt,
						},
						highlight = {
							type = "select", order = 223, style = "dropdown",
							name = L["Highlight"],
							desc = L["Config/Loot/HighlightDesc"],
							values = qualityKeys,
							get = function(info)
								return NekoLoot.db.profile.loot.highlight + 1
							end,
							set = function(info, value) 
								NekoLoot.db.profile.loot.highlight = value - 1
							end,
						},

						heightColorSpace = {
							type = "header", order = 230,
							name = "",
						},
						height = {
							type = "range", order = 231,
							name = L["Height"],
							desc = L["Config/Loot/HeightDesc"],
							min = 10, max = 100, step = 1,
							get = getOpt,
							set = setOpt,
						},
						backgroundColor = {
							type = "color", order = 232, hasAlpha = true,
							name = L["BackgroundColor"],
							get = getColor,
							set = setColor,
						},
						buttonBorderColor = {
							type = "color", order = 233, hasAlpha = true,
							name = L["Config/Loot/ButtonBorderColor"],
							desc = L["Config/Loot/ButtonBorderColorDesc"],
							get = getColor,
							set = setColor,
						},
						
						lootHighlightHeader = {
							type = "header", order = 240,
							name = L["Config/Loot/Highlight"],
						},
						highlightButton = {
							type = "toggle", order = 241,
							name = L["Config/Loot/HighlightButton"],
							desc = L["Config/Loot/HighlightButtonDesc"],
							get = getOpt,
							set = setOpt,
						},
						highlightFrame = {
							type = "toggle", order = 242,
							name = L["Config/Loot/HighlightFrame"],
							desc = L["Config/Loot/HighlightFrameDesc"],
							get = getOpt,
							set = setOpt,
						},
					},
				},
				---- loot

				lootMode = {
					type = "group", inline = false,
					name = L["ConfigFrame/LootMode"],
					desc = L["ConfigFrame/LootModeDesc"],
					args = {
						warning = {
							type = "header", order = 300,
							name = RED_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/Warning"],
						},
						warningText = {
							type = "description", order = 301, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/WarningText"],
						},
						
						lootModeSpace1 = {
							type = "description", order = 310, fontSize = "large",
							name = " ",
						},
						
						fast = {
							type = "toggle", order = 340, width = "double",
							name = L["ConfigFrame/LootMode/Fast"],
							get = function(info) return NekoLoot.db.profile.lootMode.fast end,
							set = function(info, value)
								NekoLoot.db.profile.lootMode.extraFast = false
								NekoLoot.db.profile.lootMode.fast = value
								NekoLoot:ChangeLootOpen(info[#info], value)
							end,
						},
						fastDesc = {
							type = "description", order = 341, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/FastDesc"],
						},
						extraFast = {
							type = "toggle", order = 345, width = "double",
							name = L["ConfigFrame/LootMode/ExtraFast"],
							get = function(info) return NekoLoot.db.profile.lootMode.extraFast end,
							set = function(info, value)
								NekoLoot.db.profile.lootMode.fast = false
								NekoLoot.db.profile.lootMode.extraFast = value
								NekoLoot:ChangeLootOpen(info[#info], value)
							end,
						},
						extraFastDesc = {
							type = "description", order = 346, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/ExtraFastDesc"],
						},

						lootModeSpace2 = {
							type = "description", order = 350, fontSize = "large",
							name = " ",
						},

						collapse = {
							type = "toggle", order = 351, width = "double",
							name = L["ConfigFrame/LootMode/Collapse"],
							get = function(info) return NekoLoot.db.profile.lootMode.collapse end,
							set = function(info, value)
								NekoLoot.db.profile.lootMode.fastCollapse = false
								NekoLoot.db.profile.lootMode.collapse = value
								NekoLoot:ChangeLootSlotCleared(info[#info], value)
							end,
						},
						collapseDesc = {
							type = "description", order = 352, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/CollapseDesc"],
						},
						fastCollapse = {
							type = "toggle", order = 355, width = "double",
							name = L["ConfigFrame/LootMode/FastCollapse"],
							get = function(info) return NekoLoot.db.profile.lootMode.fastCollapse end,
							set = function(info, value)
								NekoLoot.db.profile.lootMode.collapse = false
								NekoLoot.db.profile.lootMode.fastCollapse = value
								NekoLoot:ChangeLootSlotCleared(info[#info], value)
							end,
						},
						fastCollapseDesc = {
							type = "description", order = 356, fontSize = "medium",
							name = NORMAL_FONT_COLOR_CODE .. L["ConfigFrame/LootMode/FastCollapseDesc"],
						},
					},
				},
				----- lootMode

				config = {
					type = "execute", guiHidden = true,
					name = L["ConfigFrame/Config"],
					desc = L["ConfigFrame/ConfigDesc"],
					func = function()
						if AceConfDialog.OpenFrames["NekoLoot"] then
							AceConfDialog:Close("NekoLoot")
						else
							AceConfDialog:Open("NekoLoot")
						end
					end,
				},
				---- config

				enable = {
					type = "execute", guiHidden = true,
					name = L["ConfigFrame/Enable"],
					desc = L["ConfigFrame/EnableDesc"],
					func = function() self:OnEnable(); end,
				},
				---- enable
				
				disable = {
					type = "execute", guiHidden = true,
					name = L["ConfigFrame/Disable"],
					desc = L["ConfigFrame/DisableDesc"],
					func = function() self:OnDisable() end,
				},
				---- disable
			},
		}
	end
	for modName, moduleOpt in pairs(moduleOptions) do
		options.args[modName] = moduleOpt()
	end
	return options
end


------
-- Set up addon database. Register option tables to Blizzard config menu.
function NekoLoot:SetupOptions()
	for index = 0, 7 do
		qualityKeys[index + 1] = 
			ITEM_QUALITY_COLORS[index].hex.._G["ITEM_QUALITY"..tostring(index).."_DESC"].."|r"
	end

	-- register so config changes can get
	AceConfReg:RegisterOptionsTable("NekoLoot", getOptions)

	AceConf:RegisterOptionsTable("NekoLoot", getOptions, {"NekoLoot", "nl"})
	
	AceConfDialog:AddToBlizOptions("NekoLoot", "NekoLoot", nil)

	-- insert wowace profile handling
	getOptions().args.profiles = AceDBOpt:GetOptionsTable(self.db)
	
	AceConfDialog:AddToBlizOptions("NekoLoot", L["Frame"], "NekoLoot", "frame")
	AceConfDialog:AddToBlizOptions("NekoLoot", L["Loot"], "NekoLoot", "loot")
	AceConfDialog:AddToBlizOptions("NekoLoot", L["LootMode"], "NekoLoot", "lootMode")

	AceConfDialog:AddToBlizOptions("NekoLoot", "Profiles", "NekoLoot", "profiles")
end


------
-- Registers a module to options dialog and table.
--
-- name				: name of module
-- getOptTable		: getter of modules option table
-- displayName		: name of module to display in options dialog
function NekoLoot:RegisterModuleOptions(name, getOptTable, displayName)
	moduleOptions[name] = getOptTable
	AceConfDialog:AddToBlizOptions("NekoLoot", displayName, "NekoLoot", name)
end
